#!/usr/bin/env sh

RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[0;33m'
BLUE='\033[0;34m'
NC='\033[39m' # No Color

die() {
    echo -e '[ '"${RED}$1${NC}"' ]'
    exit 1
}

warning() {
    echo -e '[ '"${YELLOW}$1${NC}"' ]'
}

notify() {
    echo -e '[ '"${BLUE}$1${NC}"' ]'
}

success() {
    echo -e '[ '"${GREEN}$1${NC}"' ]'
}

warning "Starting at $(date -u)"
rm -rf repo || die "Couldn't remove files"

if [ -n "$OPENWRT_GIT_URL" ]; then
    git config --global user.email "${OPENWRT_GIT_EMAIL:-openwrt@example.com}" || die "Couldn't set email"
    git config --global user.name "${OPENWRT_GIT_NAME:-OpenWRT Git bot}" || die "Couldn't set name"
    git clone -q "$OPENWRT_GIT_URL" repo || die "Couldn't clone repo"
    cd repo || die "Couldn't enter repo dir"

    notify "Entering main loop at $(date -u)"
    while true; do
        git reset -q --hard || die "Couldn't reset repo"
        git pull -q || die "Couldn't pull repo"

        LC_ALL=C.UTF-8 rsync -acxq --delete-after --exclude '*-opkg' -e "ssh -p ${OPENWRT_SSH_PORT:-22}" "${OPENWRT_SSH_HOST:-root@192.168.0.1}:/overlay/upper/" overlay/ || die "Couldn't rsync config"
        find overlay/ -type d -empty -exec touch {}/.gitkeep \;

        git add -A > /dev/null || die "Couldn't add files"
        if [ -n "$(git status -su)" ]; then
            warning "Updating at $(date -u)"
            git commit -q -m OpenWRT || die "Couldn't commit changes"
            git push -q --force origin HEAD:${OPENWRT_GIT_BRANCH:-remote} || die "Couldn't push changes"
            success "Updated at $(date -u)"
        fi

        sleep ${OPENWRT_SYNC_INTERVAL:-10} || die "Couldn't sleep correctly"
    done
    cd -
    die "Somebody has escaped from endless cycle"
else
    die "OPENWRT_GIT_URL is not configured"
fi
